# -*- coding: utf-8 -*-
from __future__ import print_function
from __future__ import unicode_literals
import sys
import os

sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from mn_translit import (
    latin_to_cyrillic, 
    cyrillic_to_latin, 
    number_to_words, 
    words_to_number,
    transliterate
)


def test_latin_to_cyrillic():
    assert latin_to_cyrillic("Mongolia") == "Монголиа"
    assert latin_to_cyrillic("Sain baina uu?") == "Сайн байна уу?"
    assert latin_to_cyrillic("Sainai") == "Сайнай"
    assert latin_to_cyrillic("oilgokh") == "ойлгох"
    assert latin_to_cyrillic("nom") == "ном"
    assert latin_to_cyrillic("ger") == "гэр"
    assert latin_to_cyrillic("khün") == "хүн"
    assert latin_to_cyrillic("tsagaan") == "цагаан"
    print("✓ Latin to Cyrillic tests passed")


def test_cyrillic_to_latin():
    assert cyrillic_to_latin("Монгол") == "Mongol"
    assert cyrillic_to_latin("Улаанбаатар") == "Ulaanbaatar"
    assert cyrillic_to_latin("сайн") == "sayn"
    assert cyrillic_to_latin("ном") == "nom"
    assert cyrillic_to_latin("гэр") == "ger"
    assert cyrillic_to_latin("хүн") == "khün"
    assert cyrillic_to_latin("цагаан") == "tsagaan"
    print("✓ Cyrillic to Latin tests passed")


def test_diphthongs():
    assert latin_to_cyrillic("ai") == "ай"
    assert latin_to_cyrillic("ei") == "эй"
    assert latin_to_cyrillic("ii") == "ий"
    assert latin_to_cyrillic("oi") == "ой"
    assert latin_to_cyrillic("Sainai") == "Сайнай"
    assert latin_to_cyrillic("oilgokh") == "ойлгох"
    print("✓ Diphthong tests passed")


def test_number_to_words():
    assert number_to_words(0) == "тэг"
    assert number_to_words(1) == "нэг"
    assert number_to_words(21) == "хорин нэг"
    assert number_to_words(31) == "гучин нэг"
    assert number_to_words(41) == "дөчин нэг"
    assert number_to_words(51) == "тавин нэг"
    assert number_to_words(61) == "жаран нэг"
    assert number_to_words(71) == "далан нэг"
    assert number_to_words(81) == "наян нэг"
    assert number_to_words(91) == "ерэн нэг"
    assert number_to_words(100) == "зуун"
    assert number_to_words(111) == "зуун арав нэг"
    assert number_to_words(230) == "хоёр зуун гучин"
    assert number_to_words(1000) == "мянга"
    assert number_to_words(2024) == "хоёр мянга хорин дөрөв"
    print("✓ Number to words tests passed")


def test_words_to_number():
    assert words_to_number("тэг") == 0
    assert words_to_number("нэг") == 1
    assert words_to_number("мянга") == 1000
    assert words_to_number("хорин нэг") == 21
    assert words_to_number("гучин нэг") == 31
    assert words_to_number("зуун арав нэг") == 111
    assert words_to_number("хоёр зуун гучин") == 230
    assert words_to_number("хоёр мянга хорин дөрөв") == 2024
    print("✓ Words to number tests passed")


def test_trans_num_latin_to_cyrillic():
    result = latin_to_cyrillic("Bi 21 nom baina", trans_num=True)
    assert "хорин нэг" in result
    assert "21" not in result
    
    result = latin_to_cyrillic("On 2024", trans_num=True)
    assert "хоёр мянга хорин дөрөв" in result
    
    result = latin_to_cyrillic("Une ni 100 tugrug", trans_num=True)
    assert "зуун" in result
    
    result = latin_to_cyrillic("Mongol khun")
    assert result == latin_to_cyrillic("Mongol khun", trans_num=True)
    
    print("✓ Latin to Cyrillic with trans_num tests passed")


def test_trans_num_false_by_default():
    result = latin_to_cyrillic("Bi 21 nom baina")
    assert "21" in result
    assert "хорин нэг" not in result
    print("✓ trans_num=False by default test passed")


def test_major_number_scales():
    assert number_to_words(100) == "зуун"
    assert number_to_words(1000) == "мянга"
    assert number_to_words(10000) == "түм"
    assert number_to_words(100000) == "бум"
    assert number_to_words(1000000) == "сая"
    print("✓ Major number scales tests passed")


def test_transliterate_function():
    assert transliterate("Mongolia", to_script='cyrillic') == "Монголиа"
    assert transliterate("Монгол", to_script='latin') == "Mongol"
    
    result = transliterate("On 2024", to_script='cyrillic', trans_num=True)
    assert "хоёр мянга хорин дөрөв" in result
    
    print("✓ Transliterate function tests passed")


def test_edge_cases():
    assert latin_to_cyrillic("") == ""
    assert cyrillic_to_latin("") == ""
    assert latin_to_cyrillic("123") == "123"
    assert latin_to_cyrillic("!@#$%") == "!@#$%"
    print("✓ Edge case tests passed")


def run_all_tests():
    print("=" * 60)
    print("Running mn-translit Test Suite")
    print("=" * 60)
    print()
    
    tests = [
        test_latin_to_cyrillic,
        test_cyrillic_to_latin,
        test_diphthongs,
        test_number_to_words,
        test_words_to_number,
        test_trans_num_latin_to_cyrillic,
        test_trans_num_false_by_default,
        test_major_number_scales,
        test_transliterate_function,
        test_edge_cases,
    ]
    
    failed = 0
    for test in tests:
        try:
            test()
        except AssertionError as e:
            print("✗ {} failed: {}".format(test.__name__, str(e)))
            failed += 1
        except Exception as e:
            print("✗ {} error: {}".format(test.__name__, str(e)))
            failed += 1
    
    print()
    print("=" * 60)
    if failed == 0:
        print("All tests passed! ✓")
        print("=" * 60)
        return 0
    else:
        print("{} test(s) failed!".format(failed))
        print("=" * 60)
        return 1


if __name__ == '__main__':
    sys.exit(run_all_tests())



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

__version__ = '1.0.0'
__author__ = 'Your Name'
__license__ = 'MIT'

from .translit import (
    latin_to_cyrillic,
    cyrillic_to_latin,
    transliterate,
    number_to_words,
    words_to_number,
    MongolianTransliterator
)

__all__ = [
    'latin_to_cyrillic',
    'cyrillic_to_latin',
    'transliterate',
    'number_to_words',
    'words_to_number',
    'MongolianTransliterator'
]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals


class MongolianTransliterator(object):
    
    LATIN_TO_CYRILLIC_MULTI = [
        ('kh', 'х'), ('ts', 'ц'), ('ch', 'ч'), ('sh', 'ш'),
        ('yo', 'ё'), ('yu', 'ю'), ('ya', 'я'), ('ye', 'е'), ('zh', 'ж'),
        ('ai', 'ай'), ('ei', 'эй'), ('ii', 'ий'), ('oi', 'ой'),
        ('Kh', 'Х'), ('KH', 'Х'), ('Ts', 'Ц'), ('TS', 'Ц'),
        ('Ch', 'Ч'), ('CH', 'Ч'), ('Sh', 'Ш'), ('SH', 'Ш'),
        ('Yo', 'Ё'), ('YO', 'Ё'), ('Yu', 'Ю'), ('YU', 'Ю'),
        ('Ya', 'Я'), ('YA', 'Я'), ('Ye', 'Е'), ('YE', 'Е'),
        ('Zh', 'Ж'), ('ZH', 'Ж'),
    ]
    
    LATIN_TO_CYRILLIC_SINGLE = {
        'a': 'а', 'b': 'б', 'v': 'в', 'g': 'г', 'd': 'д', 'e': 'э',
        'z': 'з', 'i': 'и', 'y': 'й', 'k': 'к', 'l': 'л', 'm': 'м',
        'n': 'н', 'o': 'о', 'p': 'п', 'r': 'р', 's': 'с', 't': 'т',
        'u': 'у', 'f': 'ф', 'h': 'х', 'c': 'ц', 'w': 'в',
        'A': 'А', 'B': 'Б', 'V': 'В', 'G': 'Г', 'D': 'Д', 'E': 'Э',
        'Z': 'З', 'I': 'И', 'Y': 'Й', 'K': 'К', 'L': 'Л', 'M': 'М',
        'N': 'Н', 'O': 'О', 'P': 'П', 'R': 'Р', 'S': 'С', 'T': 'Т',
        'U': 'У', 'F': 'Ф', 'H': 'Х', 'C': 'Ц', 'W': 'В',
        'ö': 'ө', 'ü': 'ү', 'Ö': 'Ө', 'Ü': 'Ү',
    }
    
    CYRILLIC_TO_LATIN = {
        'а': 'a', 'б': 'b', 'в': 'v', 'г': 'g', 'д': 'd', 'е': 'ye',
        'ё': 'yo', 'ж': 'zh', 'з': 'z', 'и': 'i', 'й': 'y', 'к': 'k',
        'л': 'l', 'м': 'm', 'н': 'n', 'о': 'o', 'ө': 'ö', 'п': 'p',
        'р': 'r', 'с': 's', 'т': 't', 'у': 'u', 'ү': 'ü', 'ф': 'f',
        'х': 'kh', 'ц': 'ts', 'ч': 'ch', 'ш': 'sh', 'щ': 'shch',
        'ъ': '', 'ы': 'y', 'ь': '', 'э': 'e', 'ю': 'yu', 'я': 'ya',
        'А': 'A', 'Б': 'B', 'В': 'V', 'Г': 'G', 'Д': 'D', 'Е': 'Ye',
        'Ё': 'Yo', 'Ж': 'Zh', 'З': 'Z', 'И': 'I', 'Й': 'Y', 'К': 'K',
        'Л': 'L', 'М': 'M', 'Н': 'N', 'О': 'O', 'Ө': 'Ö', 'П': 'P',
        'Р': 'R', 'С': 'S', 'Т': 'T', 'У': 'U', 'Ү': 'Ü', 'Ф': 'F',
        'Х': 'Kh', 'Ц': 'Ts', 'Ч': 'Ch', 'Ш': 'Sh', 'Щ': 'Shch',
        'Ъ': '', 'Ы': 'Y', 'Ь': '', 'Э': 'E', 'Ю': 'Yu', 'Я': 'Ya',
    }
    
    NUMBERS = {
        0: 'тэг', 1: 'нэг', 2: 'хоёр', 3: 'гурав', 4: 'дөрөв', 5: 'тав',
        6: 'зургаа', 7: 'долоо', 8: 'найм', 9: 'ес', 10: 'арав',
        20: 'хорин', 30: 'гучин', 40: 'дөчин', 50: 'тавин', 60: 'жаран',
        70: 'далан', 80: 'наян', 90: 'ерэн', 100: 'зуун', 1000: 'мянга',
        10000: 'түм', 100000: 'бум', 1000000: 'сая', 1000000000: 'тэрбум',
        1000000000000: 'их наяд'
    }
    
    WORD_TO_NUMBER = {v: k for k, v in NUMBERS.items()}
    
    def _convert_numbers_in_text(self, text, to_words=True):
        import re
        if to_words:
            def replace_number(match):
                num = int(match.group())
                try:
                    return self.number_to_words(num)
                except:
                    return match.group()
            return re.sub(r'\d+', replace_number, text)
        else:
            words = text.split()
            result = []
            i = 0
            while i < len(words):
                word_lower = words[i].lower()
                if word_lower in self.WORD_TO_NUMBER or (i + 1 < len(words) and word_lower + ' ' + words[i + 1].lower() in ' '.join([w.lower() for w in words[i:i+3]])):
                    try:
                        num_words = []
                        j = i
                        while j < len(words) and words[j].lower() in self.WORD_TO_NUMBER:
                            num_words.append(words[j])
                            j += 1
                        if num_words:
                            num_text = ' '.join(num_words)
                            try:
                                num = self.words_to_number(num_text)
                                result.append(str(num))
                                i = j
                                continue
                            except:
                                pass
                    except:
                        pass
                result.append(words[i])
                i += 1
            return ' '.join(result)
    
    def latin_to_cyrillic(self, text, trans_num=False):
        if not text:
            return text
        
        if trans_num:
            text = self._convert_numbers_in_text(text, to_words=True)
        
        result, i = [], 0
        while i < len(text):
            matched = False
            if i + 2 <= len(text):
                substring = text[i:i+2]
                for latin, cyrillic in self.LATIN_TO_CYRILLIC_MULTI:
                    if substring == latin:
                        result.append(cyrillic)
                        i += 2
                        matched = True
                        break
            
            if not matched:
                char = text[i]
                result.append(self.LATIN_TO_CYRILLIC_SINGLE.get(char, char))
                i += 1
        
        return ''.join(result)
    
    def cyrillic_to_latin(self, text, trans_num=False):
        if not text:
            return text
        
        result = ''.join([self.CYRILLIC_TO_LATIN.get(char, char) for char in text])
        
        if trans_num:
            result = self._convert_numbers_in_text(result, to_words=False)
        
        return result
    
    def number_to_words(self, num):
        if not isinstance(num, int) or num < 0:
            raise ValueError("Number must be a positive integer")
        
        if num in self.NUMBERS:
            return self.NUMBERS[num]
        
        if num < 20:
            return self.NUMBERS[10] + ' ' + self.NUMBERS[num % 10]
        
        if num < 100:
            tens, ones = (num // 10) * 10, num % 10
            return self.NUMBERS[tens] if ones == 0 else self.NUMBERS[tens] + ' ' + self.NUMBERS[ones]
        
        if num < 1000:
            hundreds, remainder = num // 100, num % 100
            if hundreds == 1:
                result = self.NUMBERS[100]
            else:
                result = self.NUMBERS[hundreds] + ' ' + self.NUMBERS[100]
            if remainder >= 10:
                result += ' ' + self.number_to_words(remainder)
            elif remainder > 0:
                result += ' ' + self.NUMBERS[remainder]
            return result
        
        if num < 10000:
            thousands, remainder = num // 1000, num % 1000
            result = self.NUMBERS[thousands] + ' ' + self.NUMBERS[1000]
            return result if remainder == 0 else result + ' ' + self.number_to_words(remainder)
        
        if num < 100000:
            ten_thousands, remainder = num // 10000, num % 10000
            if ten_thousands == 1:
                result = self.NUMBERS[10000]
            else:
                result = self.number_to_words(ten_thousands) + ' ' + self.NUMBERS[10000]
            return result if remainder == 0 else result + ' ' + self.number_to_words(remainder)
        
        if num < 1000000:
            hundred_thousands, remainder = num // 100000, num % 100000
            if hundred_thousands == 1:
                result = self.NUMBERS[100000]
            else:
                result = self.number_to_words(hundred_thousands) + ' ' + self.NUMBERS[100000]
            return result if remainder == 0 else result + ' ' + self.number_to_words(remainder)
        
        if num < 1000000000:
            millions, remainder = num // 1000000, num % 1000000
            result = self.number_to_words(millions) + ' ' + self.NUMBERS[1000000]
            return result if remainder == 0 else result + ' ' + self.number_to_words(remainder)
        
        if num < 1000000000000:
            billions, remainder = num // 1000000000, num % 1000000000
            result = self.number_to_words(billions) + ' ' + self.NUMBERS[1000000000]
            return result if remainder == 0 else result + ' ' + self.number_to_words(remainder)
        
        trillions, remainder = num // 1000000000000, num % 1000000000000
        result = self.number_to_words(trillions) + ' ' + self.NUMBERS[1000000000000]
        return result if remainder == 0 else result + ' ' + self.number_to_words(remainder)
    
    def words_to_number(self, text):
        words = text.strip().lower().split()
        if len(words) == 1 and words[0] in self.WORD_TO_NUMBER:
            return self.WORD_TO_NUMBER[words[0]]
        
        total, current = 0, 0
        i = 0
        
        while i < len(words):
            word = words[i]
            
            if word == 'их' and i + 1 < len(words) and words[i + 1] == 'наяд':
                num = self.WORD_TO_NUMBER.get('их наяд', 1000000000000)
                total += (current if current > 0 else 1) * num
                current = 0
                i += 2
                continue
            
            if word not in self.WORD_TO_NUMBER:
                raise ValueError("Unknown word: {}".format(word))
            
            num = self.WORD_TO_NUMBER[word]
            
            if num >= 1000:
                total += (current if current > 0 else 1) * num
                current = 0
            elif num == 100:
                if current == 0:
                    current = 100
                else:
                    current *= num
            else:
                current += num
            
            i += 1
        
        return total + current
    
    def transliterate(self, text, to_script='cyrillic', trans_num=False):
        if to_script.lower() in ['cyrillic', 'cyr', 'c']:
            return self.latin_to_cyrillic(text, trans_num=trans_num)
        elif to_script.lower() in ['latin', 'lat', 'l']:
            return self.cyrillic_to_latin(text, trans_num=trans_num)
        raise ValueError("to_script must be 'cyrillic' or 'latin'")


_transliterator = MongolianTransliterator()

def latin_to_cyrillic(text, trans_num=False):
    return _transliterator.latin_to_cyrillic(text, trans_num=trans_num)

def cyrillic_to_latin(text, trans_num=False):
    return _transliterator.cyrillic_to_latin(text, trans_num=trans_num)

def number_to_words(num):
    return _transliterator.number_to_words(num)

def words_to_number(text):
    return _transliterator.words_to_number(text)

def transliterate(text, to_script='cyrillic', trans_num=False):
    return _transliterator.transliterate(text, to_script=to_script, trans_num=trans_num)
